"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class JobStatus(str, Enum):
    r"""The status of the job."""

    DRAFT = "draft"
    INTERNAL = "internal"
    PUBLISHED = "published"
    COMPLETED = "completed"
    PLACED = "placed"
    ON_HOLD = "on-hold"
    PRIVATE = "private"
    ACCEPTING_CANDIDATES = "accepting_candidates"
    OPEN = "open"
    CLOSED = "closed"
    ARCHIVED = "archived"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from apideck_unify import models, utils
from apideck_unify._hooks import HookContext
from apideck_unify.types import Nullable, OptionalNullable, UNSET
from apideck_unify.utils import get_security_from_env
from apideck_unify.utils.unmarshal_json_response import unmarshal_json_response
from datetime import date
from jsonpath import JSONPath
from typing import Any, Dict, List, Mapping, Optional, Union


class Companies(BaseSDK):
    def list(
        self,
        *,
        raw: Optional[bool] = False,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        cursor: OptionalNullable[str] = UNSET,
        limit: Optional[int] = 20,
        filter_: Optional[
            Union[models.CompaniesFilter, models.CompaniesFilterTypedDict]
        ] = None,
        sort: Optional[
            Union[models.CompaniesSort, models.CompaniesSortTypedDict]
        ] = None,
        pass_through: Optional[Dict[str, Any]] = None,
        fields: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Optional[models.CrmCompaniesAllResponse]:
        r"""List companies

        List companies

        :param raw: Include raw response. Mostly used for debugging purposes
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param cursor: Cursor to start from. You can find cursors for next/previous pages in the meta.cursors property of the response.
        :param limit: Number of results to return. Minimum 1, Maximum 200, Default 20
        :param filter_: Apply filters
        :param sort: Apply sorting
        :param pass_through: Optional unmapped key/values that will be passed through to downstream as query parameters. Ie: ?pass_through[search]=leads becomes ?search=leads
        :param fields: The 'fields' parameter allows API users to specify the fields they want to include in the API response. If this parameter is not present, the API will return all available fields. If this parameter is present, only the fields specified in the comma-separated string will be included in the response. Nested properties can also be requested by using a dot notation. <br /><br />Example: `fields=name,email,addresses.city`<br /><br />In the example above, the response will only include the fields \"name\", \"email\" and \"addresses.city\". If any other fields are available, they will be excluded.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CrmCompaniesAllRequest(
            raw=raw,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            cursor=cursor,
            limit=limit,
            filter_=utils.get_pydantic_model(filter_, Optional[models.CompaniesFilter]),
            sort=utils.get_pydantic_model(sort, Optional[models.CompaniesSort]),
            pass_through=pass_through,
            fields=fields,
        )

        req = self._build_request(
            method="GET",
            path="/crm/companies",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.CrmCompaniesAllGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="crm.companiesAll",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        def next_func() -> Optional[models.CrmCompaniesAllResponse]:
            body = utils.unmarshal_json(http_res.text, Union[Dict[Any, Any], List[Any]])
            next_cursor = JSONPath("$.meta.cursors.next").parse(body)

            if len(next_cursor) == 0:
                return None

            next_cursor = next_cursor[0]
            if next_cursor is None:
                return None

            return self.list(
                raw=raw,
                consumer_id=consumer_id,
                app_id=app_id,
                service_id=service_id,
                cursor=next_cursor,
                limit=limit,
                filter_=filter_,
                sort=sort,
                pass_through=pass_through,
                fields=fields,
                retries=retries,
            )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.CrmCompaniesAllResponse(
                get_companies_response=unmarshal_json_response(
                    Optional[models.GetCompaniesResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
                next=next_func,
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.BadRequestResponseData, http_res
            )
            raise models.BadRequestResponse(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.UnauthorizedResponseData, http_res
            )
            raise models.UnauthorizedResponse(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                models.PaymentRequiredResponseData, http_res
            )
            raise models.PaymentRequiredResponse(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.NotFoundResponseData, http_res
            )
            raise models.NotFoundResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.UnprocessableResponseData, http_res
            )
            raise models.UnprocessableResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "default", "application/json"):
            return models.CrmCompaniesAllResponse(
                unexpected_error_response=unmarshal_json_response(
                    Optional[models.UnexpectedErrorResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
                next=next_func,
            )

        raise models.APIError("Unexpected response received", http_res)

    async def list_async(
        self,
        *,
        raw: Optional[bool] = False,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        cursor: OptionalNullable[str] = UNSET,
        limit: Optional[int] = 20,
        filter_: Optional[
            Union[models.CompaniesFilter, models.CompaniesFilterTypedDict]
        ] = None,
        sort: Optional[
            Union[models.CompaniesSort, models.CompaniesSortTypedDict]
        ] = None,
        pass_through: Optional[Dict[str, Any]] = None,
        fields: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Optional[models.CrmCompaniesAllResponse]:
        r"""List companies

        List companies

        :param raw: Include raw response. Mostly used for debugging purposes
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param cursor: Cursor to start from. You can find cursors for next/previous pages in the meta.cursors property of the response.
        :param limit: Number of results to return. Minimum 1, Maximum 200, Default 20
        :param filter_: Apply filters
        :param sort: Apply sorting
        :param pass_through: Optional unmapped key/values that will be passed through to downstream as query parameters. Ie: ?pass_through[search]=leads becomes ?search=leads
        :param fields: The 'fields' parameter allows API users to specify the fields they want to include in the API response. If this parameter is not present, the API will return all available fields. If this parameter is present, only the fields specified in the comma-separated string will be included in the response. Nested properties can also be requested by using a dot notation. <br /><br />Example: `fields=name,email,addresses.city`<br /><br />In the example above, the response will only include the fields \"name\", \"email\" and \"addresses.city\". If any other fields are available, they will be excluded.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CrmCompaniesAllRequest(
            raw=raw,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            cursor=cursor,
            limit=limit,
            filter_=utils.get_pydantic_model(filter_, Optional[models.CompaniesFilter]),
            sort=utils.get_pydantic_model(sort, Optional[models.CompaniesSort]),
            pass_through=pass_through,
            fields=fields,
        )

        req = self._build_request_async(
            method="GET",
            path="/crm/companies",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.CrmCompaniesAllGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="crm.companiesAll",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        def next_func() -> Optional[models.CrmCompaniesAllResponse]:
            body = utils.unmarshal_json(http_res.text, Union[Dict[Any, Any], List[Any]])
            next_cursor = JSONPath("$.meta.cursors.next").parse(body)

            if len(next_cursor) == 0:
                return None

            next_cursor = next_cursor[0]
            if next_cursor is None:
                return None

            return self.list(
                raw=raw,
                consumer_id=consumer_id,
                app_id=app_id,
                service_id=service_id,
                cursor=next_cursor,
                limit=limit,
                filter_=filter_,
                sort=sort,
                pass_through=pass_through,
                fields=fields,
                retries=retries,
            )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.CrmCompaniesAllResponse(
                get_companies_response=unmarshal_json_response(
                    Optional[models.GetCompaniesResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
                next=next_func,
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.BadRequestResponseData, http_res
            )
            raise models.BadRequestResponse(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.UnauthorizedResponseData, http_res
            )
            raise models.UnauthorizedResponse(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                models.PaymentRequiredResponseData, http_res
            )
            raise models.PaymentRequiredResponse(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.NotFoundResponseData, http_res
            )
            raise models.NotFoundResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.UnprocessableResponseData, http_res
            )
            raise models.UnprocessableResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "default", "application/json"):
            return models.CrmCompaniesAllResponse(
                unexpected_error_response=unmarshal_json_response(
                    Optional[models.UnexpectedErrorResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
                next=next_func,
            )

        raise models.APIError("Unexpected response received", http_res)

    def create(
        self,
        *,
        name: Nullable[str],
        raw: Optional[bool] = False,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        owner_id: OptionalNullable[str] = UNSET,
        image: OptionalNullable[str] = UNSET,
        description: OptionalNullable[str] = UNSET,
        vat_number: OptionalNullable[str] = UNSET,
        currency: OptionalNullable[models.Currency] = UNSET,
        status: OptionalNullable[str] = UNSET,
        fax: OptionalNullable[str] = UNSET,
        annual_revenue: OptionalNullable[str] = UNSET,
        number_of_employees: OptionalNullable[str] = UNSET,
        industry: OptionalNullable[str] = UNSET,
        ownership: OptionalNullable[str] = UNSET,
        sales_tax_number: OptionalNullable[str] = UNSET,
        payee_number: OptionalNullable[str] = UNSET,
        abn_or_tfn: OptionalNullable[str] = UNSET,
        abn_branch: OptionalNullable[str] = UNSET,
        acn: OptionalNullable[str] = UNSET,
        first_name: OptionalNullable[str] = UNSET,
        last_name: OptionalNullable[str] = UNSET,
        bank_accounts: Optional[
            Union[List[models.BankAccount1], List[models.BankAccount1TypedDict]]
        ] = None,
        websites: Optional[
            Union[List[models.Website], List[models.WebsiteTypedDict]]
        ] = None,
        addresses: Optional[
            Union[List[models.Address], List[models.AddressTypedDict]]
        ] = None,
        social_links: Optional[
            Union[List[models.SocialLink], List[models.SocialLinkTypedDict]]
        ] = None,
        phone_numbers: Optional[
            Union[List[models.PhoneNumber], List[models.PhoneNumberTypedDict]]
        ] = None,
        emails: Optional[Union[List[models.Email], List[models.EmailTypedDict]]] = None,
        row_type: Optional[
            Union[models.CompanyRowType, models.CompanyRowTypeTypedDict]
        ] = None,
        custom_fields: Optional[
            Union[List[models.CustomField], List[models.CustomFieldTypedDict]]
        ] = None,
        tags: OptionalNullable[List[str]] = UNSET,
        read_only: OptionalNullable[bool] = UNSET,
        salutation: OptionalNullable[str] = UNSET,
        birthday: OptionalNullable[date] = UNSET,
        pass_through: Optional[
            Union[List[models.PassThroughBody], List[models.PassThroughBodyTypedDict]]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CrmCompaniesAddResponse:
        r"""Create company

        Create company

        :param name: Name of the company
        :param raw: Include raw response. Mostly used for debugging purposes
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param owner_id: Owner ID
        :param image: The Image URL of the company
        :param description: A description of the company
        :param vat_number: The VAT number of the company
        :param currency: Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217).
        :param status: The status of the company
        :param fax: The fax number of the company
        :param annual_revenue: The annual revenue of the company
        :param number_of_employees: Number of employees
        :param industry: The industry represents the type of business the company is in.
        :param ownership: The ownership indicates the type of ownership of the company.
        :param sales_tax_number: A sales tax number is a unique number that identifies a company for tax purposes.
        :param payee_number: A payee number is a unique number that identifies a payee for tax purposes.
        :param abn_or_tfn: An ABN is necessary for operating a business, while a TFN (Tax File Number) is required for any person working in Australia.
        :param abn_branch: An ABN Branch (also known as a GST Branch) is used if part of your business needs to account for GST separately from its parent entity.
        :param acn: The Australian Company Number (ACN) is a nine digit number with the last digit being a check digit calculated using a modified modulus 10 calculation. ASIC has adopted a convention of always printing and displaying the ACN in the format XXX XXX XXX; three blocks of three characters, each block separated by a blank.
        :param first_name: The first name of the person.
        :param last_name: The last name of the person.
        :param bank_accounts:
        :param websites:
        :param addresses:
        :param social_links:
        :param phone_numbers:
        :param emails:
        :param row_type:
        :param custom_fields:
        :param tags:
        :param read_only: Whether the company is read-only or not
        :param salutation: A formal salutation for the person. For example, 'Mr', 'Mrs'
        :param birthday: The date of birth of the person.
        :param pass_through: The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CrmCompaniesAddRequest(
            raw=raw,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            company1=models.Company1Input(
                name=name,
                owner_id=owner_id,
                image=image,
                description=description,
                vat_number=vat_number,
                currency=currency,
                status=status,
                fax=fax,
                annual_revenue=annual_revenue,
                number_of_employees=number_of_employees,
                industry=industry,
                ownership=ownership,
                sales_tax_number=sales_tax_number,
                payee_number=payee_number,
                abn_or_tfn=abn_or_tfn,
                abn_branch=abn_branch,
                acn=acn,
                first_name=first_name,
                last_name=last_name,
                bank_accounts=utils.get_pydantic_model(
                    bank_accounts, Optional[List[models.BankAccount1]]
                ),
                websites=utils.get_pydantic_model(
                    websites, Optional[List[models.Website]]
                ),
                addresses=utils.get_pydantic_model(
                    addresses, Optional[List[models.Address]]
                ),
                social_links=utils.get_pydantic_model(
                    social_links, Optional[List[models.SocialLink]]
                ),
                phone_numbers=utils.get_pydantic_model(
                    phone_numbers, Optional[List[models.PhoneNumber]]
                ),
                emails=utils.get_pydantic_model(emails, Optional[List[models.Email]]),
                row_type=utils.get_pydantic_model(
                    row_type, Optional[models.CompanyRowType]
                ),
                custom_fields=utils.get_pydantic_model(
                    custom_fields, Optional[List[models.CustomField]]
                ),
                tags=tags,
                read_only=read_only,
                salutation=salutation,
                birthday=birthday,
                pass_through=utils.get_pydantic_model(
                    pass_through, Optional[List[models.PassThroughBody]]
                ),
            ),
        )

        req = self._build_request(
            method="POST",
            path="/crm/companies",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.CrmCompaniesAddGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.company1, False, False, "json", models.Company1Input
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="crm.companiesAdd",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return models.CrmCompaniesAddResponse(
                create_company_response=unmarshal_json_response(
                    Optional[models.CreateCompanyResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.BadRequestResponseData, http_res
            )
            raise models.BadRequestResponse(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.UnauthorizedResponseData, http_res
            )
            raise models.UnauthorizedResponse(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                models.PaymentRequiredResponseData, http_res
            )
            raise models.PaymentRequiredResponse(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.NotFoundResponseData, http_res
            )
            raise models.NotFoundResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.UnprocessableResponseData, http_res
            )
            raise models.UnprocessableResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "default", "application/json"):
            return models.CrmCompaniesAddResponse(
                unexpected_error_response=unmarshal_json_response(
                    Optional[models.UnexpectedErrorResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        raise models.APIError("Unexpected response received", http_res)

    async def create_async(
        self,
        *,
        name: Nullable[str],
        raw: Optional[bool] = False,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        owner_id: OptionalNullable[str] = UNSET,
        image: OptionalNullable[str] = UNSET,
        description: OptionalNullable[str] = UNSET,
        vat_number: OptionalNullable[str] = UNSET,
        currency: OptionalNullable[models.Currency] = UNSET,
        status: OptionalNullable[str] = UNSET,
        fax: OptionalNullable[str] = UNSET,
        annual_revenue: OptionalNullable[str] = UNSET,
        number_of_employees: OptionalNullable[str] = UNSET,
        industry: OptionalNullable[str] = UNSET,
        ownership: OptionalNullable[str] = UNSET,
        sales_tax_number: OptionalNullable[str] = UNSET,
        payee_number: OptionalNullable[str] = UNSET,
        abn_or_tfn: OptionalNullable[str] = UNSET,
        abn_branch: OptionalNullable[str] = UNSET,
        acn: OptionalNullable[str] = UNSET,
        first_name: OptionalNullable[str] = UNSET,
        last_name: OptionalNullable[str] = UNSET,
        bank_accounts: Optional[
            Union[List[models.BankAccount1], List[models.BankAccount1TypedDict]]
        ] = None,
        websites: Optional[
            Union[List[models.Website], List[models.WebsiteTypedDict]]
        ] = None,
        addresses: Optional[
            Union[List[models.Address], List[models.AddressTypedDict]]
        ] = None,
        social_links: Optional[
            Union[List[models.SocialLink], List[models.SocialLinkTypedDict]]
        ] = None,
        phone_numbers: Optional[
            Union[List[models.PhoneNumber], List[models.PhoneNumberTypedDict]]
        ] = None,
        emails: Optional[Union[List[models.Email], List[models.EmailTypedDict]]] = None,
        row_type: Optional[
            Union[models.CompanyRowType, models.CompanyRowTypeTypedDict]
        ] = None,
        custom_fields: Optional[
            Union[List[models.CustomField], List[models.CustomFieldTypedDict]]
        ] = None,
        tags: OptionalNullable[List[str]] = UNSET,
        read_only: OptionalNullable[bool] = UNSET,
        salutation: OptionalNullable[str] = UNSET,
        birthday: OptionalNullable[date] = UNSET,
        pass_through: Optional[
            Union[List[models.PassThroughBody], List[models.PassThroughBodyTypedDict]]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CrmCompaniesAddResponse:
        r"""Create company

        Create company

        :param name: Name of the company
        :param raw: Include raw response. Mostly used for debugging purposes
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param owner_id: Owner ID
        :param image: The Image URL of the company
        :param description: A description of the company
        :param vat_number: The VAT number of the company
        :param currency: Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217).
        :param status: The status of the company
        :param fax: The fax number of the company
        :param annual_revenue: The annual revenue of the company
        :param number_of_employees: Number of employees
        :param industry: The industry represents the type of business the company is in.
        :param ownership: The ownership indicates the type of ownership of the company.
        :param sales_tax_number: A sales tax number is a unique number that identifies a company for tax purposes.
        :param payee_number: A payee number is a unique number that identifies a payee for tax purposes.
        :param abn_or_tfn: An ABN is necessary for operating a business, while a TFN (Tax File Number) is required for any person working in Australia.
        :param abn_branch: An ABN Branch (also known as a GST Branch) is used if part of your business needs to account for GST separately from its parent entity.
        :param acn: The Australian Company Number (ACN) is a nine digit number with the last digit being a check digit calculated using a modified modulus 10 calculation. ASIC has adopted a convention of always printing and displaying the ACN in the format XXX XXX XXX; three blocks of three characters, each block separated by a blank.
        :param first_name: The first name of the person.
        :param last_name: The last name of the person.
        :param bank_accounts:
        :param websites:
        :param addresses:
        :param social_links:
        :param phone_numbers:
        :param emails:
        :param row_type:
        :param custom_fields:
        :param tags:
        :param read_only: Whether the company is read-only or not
        :param salutation: A formal salutation for the person. For example, 'Mr', 'Mrs'
        :param birthday: The date of birth of the person.
        :param pass_through: The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CrmCompaniesAddRequest(
            raw=raw,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            company1=models.Company1Input(
                name=name,
                owner_id=owner_id,
                image=image,
                description=description,
                vat_number=vat_number,
                currency=currency,
                status=status,
                fax=fax,
                annual_revenue=annual_revenue,
                number_of_employees=number_of_employees,
                industry=industry,
                ownership=ownership,
                sales_tax_number=sales_tax_number,
                payee_number=payee_number,
                abn_or_tfn=abn_or_tfn,
                abn_branch=abn_branch,
                acn=acn,
                first_name=first_name,
                last_name=last_name,
                bank_accounts=utils.get_pydantic_model(
                    bank_accounts, Optional[List[models.BankAccount1]]
                ),
                websites=utils.get_pydantic_model(
                    websites, Optional[List[models.Website]]
                ),
                addresses=utils.get_pydantic_model(
                    addresses, Optional[List[models.Address]]
                ),
                social_links=utils.get_pydantic_model(
                    social_links, Optional[List[models.SocialLink]]
                ),
                phone_numbers=utils.get_pydantic_model(
                    phone_numbers, Optional[List[models.PhoneNumber]]
                ),
                emails=utils.get_pydantic_model(emails, Optional[List[models.Email]]),
                row_type=utils.get_pydantic_model(
                    row_type, Optional[models.CompanyRowType]
                ),
                custom_fields=utils.get_pydantic_model(
                    custom_fields, Optional[List[models.CustomField]]
                ),
                tags=tags,
                read_only=read_only,
                salutation=salutation,
                birthday=birthday,
                pass_through=utils.get_pydantic_model(
                    pass_through, Optional[List[models.PassThroughBody]]
                ),
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/crm/companies",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.CrmCompaniesAddGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.company1, False, False, "json", models.Company1Input
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="crm.companiesAdd",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return models.CrmCompaniesAddResponse(
                create_company_response=unmarshal_json_response(
                    Optional[models.CreateCompanyResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.BadRequestResponseData, http_res
            )
            raise models.BadRequestResponse(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.UnauthorizedResponseData, http_res
            )
            raise models.UnauthorizedResponse(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                models.PaymentRequiredResponseData, http_res
            )
            raise models.PaymentRequiredResponse(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.NotFoundResponseData, http_res
            )
            raise models.NotFoundResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.UnprocessableResponseData, http_res
            )
            raise models.UnprocessableResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "default", "application/json"):
            return models.CrmCompaniesAddResponse(
                unexpected_error_response=unmarshal_json_response(
                    Optional[models.UnexpectedErrorResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        raise models.APIError("Unexpected response received", http_res)

    def get(
        self,
        *,
        id: str,
        raw: Optional[bool] = False,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        fields: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CrmCompaniesOneResponse:
        r"""Get company

        Get company

        :param id: ID of the record you are acting upon.
        :param raw: Include raw response. Mostly used for debugging purposes
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param fields: The 'fields' parameter allows API users to specify the fields they want to include in the API response. If this parameter is not present, the API will return all available fields. If this parameter is present, only the fields specified in the comma-separated string will be included in the response. Nested properties can also be requested by using a dot notation. <br /><br />Example: `fields=name,email,addresses.city`<br /><br />In the example above, the response will only include the fields \"name\", \"email\" and \"addresses.city\". If any other fields are available, they will be excluded.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CrmCompaniesOneRequest(
            id=id,
            raw=raw,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            fields=fields,
        )

        req = self._build_request(
            method="GET",
            path="/crm/companies/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.CrmCompaniesOneGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="crm.companiesOne",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.CrmCompaniesOneResponse(
                get_company_response=unmarshal_json_response(
                    Optional[models.GetCompanyResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.BadRequestResponseData, http_res
            )
            raise models.BadRequestResponse(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.UnauthorizedResponseData, http_res
            )
            raise models.UnauthorizedResponse(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                models.PaymentRequiredResponseData, http_res
            )
            raise models.PaymentRequiredResponse(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.NotFoundResponseData, http_res
            )
            raise models.NotFoundResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.UnprocessableResponseData, http_res
            )
            raise models.UnprocessableResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "default", "application/json"):
            return models.CrmCompaniesOneResponse(
                unexpected_error_response=unmarshal_json_response(
                    Optional[models.UnexpectedErrorResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        raise models.APIError("Unexpected response received", http_res)

    async def get_async(
        self,
        *,
        id: str,
        raw: Optional[bool] = False,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        fields: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CrmCompaniesOneResponse:
        r"""Get company

        Get company

        :param id: ID of the record you are acting upon.
        :param raw: Include raw response. Mostly used for debugging purposes
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param fields: The 'fields' parameter allows API users to specify the fields they want to include in the API response. If this parameter is not present, the API will return all available fields. If this parameter is present, only the fields specified in the comma-separated string will be included in the response. Nested properties can also be requested by using a dot notation. <br /><br />Example: `fields=name,email,addresses.city`<br /><br />In the example above, the response will only include the fields \"name\", \"email\" and \"addresses.city\". If any other fields are available, they will be excluded.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CrmCompaniesOneRequest(
            id=id,
            raw=raw,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            fields=fields,
        )

        req = self._build_request_async(
            method="GET",
            path="/crm/companies/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.CrmCompaniesOneGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="crm.companiesOne",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.CrmCompaniesOneResponse(
                get_company_response=unmarshal_json_response(
                    Optional[models.GetCompanyResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.BadRequestResponseData, http_res
            )
            raise models.BadRequestResponse(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.UnauthorizedResponseData, http_res
            )
            raise models.UnauthorizedResponse(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                models.PaymentRequiredResponseData, http_res
            )
            raise models.PaymentRequiredResponse(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.NotFoundResponseData, http_res
            )
            raise models.NotFoundResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.UnprocessableResponseData, http_res
            )
            raise models.UnprocessableResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "default", "application/json"):
            return models.CrmCompaniesOneResponse(
                unexpected_error_response=unmarshal_json_response(
                    Optional[models.UnexpectedErrorResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        raise models.APIError("Unexpected response received", http_res)

    def update(
        self,
        *,
        id: str,
        name: Nullable[str],
        raw: Optional[bool] = False,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        owner_id: OptionalNullable[str] = UNSET,
        image: OptionalNullable[str] = UNSET,
        description: OptionalNullable[str] = UNSET,
        vat_number: OptionalNullable[str] = UNSET,
        currency: OptionalNullable[models.Currency] = UNSET,
        status: OptionalNullable[str] = UNSET,
        fax: OptionalNullable[str] = UNSET,
        annual_revenue: OptionalNullable[str] = UNSET,
        number_of_employees: OptionalNullable[str] = UNSET,
        industry: OptionalNullable[str] = UNSET,
        ownership: OptionalNullable[str] = UNSET,
        sales_tax_number: OptionalNullable[str] = UNSET,
        payee_number: OptionalNullable[str] = UNSET,
        abn_or_tfn: OptionalNullable[str] = UNSET,
        abn_branch: OptionalNullable[str] = UNSET,
        acn: OptionalNullable[str] = UNSET,
        first_name: OptionalNullable[str] = UNSET,
        last_name: OptionalNullable[str] = UNSET,
        bank_accounts: Optional[
            Union[List[models.BankAccount1], List[models.BankAccount1TypedDict]]
        ] = None,
        websites: Optional[
            Union[List[models.Website], List[models.WebsiteTypedDict]]
        ] = None,
        addresses: Optional[
            Union[List[models.Address], List[models.AddressTypedDict]]
        ] = None,
        social_links: Optional[
            Union[List[models.SocialLink], List[models.SocialLinkTypedDict]]
        ] = None,
        phone_numbers: Optional[
            Union[List[models.PhoneNumber], List[models.PhoneNumberTypedDict]]
        ] = None,
        emails: Optional[Union[List[models.Email], List[models.EmailTypedDict]]] = None,
        row_type: Optional[
            Union[models.CompanyRowType, models.CompanyRowTypeTypedDict]
        ] = None,
        custom_fields: Optional[
            Union[List[models.CustomField], List[models.CustomFieldTypedDict]]
        ] = None,
        tags: OptionalNullable[List[str]] = UNSET,
        read_only: OptionalNullable[bool] = UNSET,
        salutation: OptionalNullable[str] = UNSET,
        birthday: OptionalNullable[date] = UNSET,
        pass_through: Optional[
            Union[List[models.PassThroughBody], List[models.PassThroughBodyTypedDict]]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CrmCompaniesUpdateResponse:
        r"""Update company

        Update company

        :param id: ID of the record you are acting upon.
        :param name: Name of the company
        :param raw: Include raw response. Mostly used for debugging purposes
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param owner_id: Owner ID
        :param image: The Image URL of the company
        :param description: A description of the company
        :param vat_number: The VAT number of the company
        :param currency: Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217).
        :param status: The status of the company
        :param fax: The fax number of the company
        :param annual_revenue: The annual revenue of the company
        :param number_of_employees: Number of employees
        :param industry: The industry represents the type of business the company is in.
        :param ownership: The ownership indicates the type of ownership of the company.
        :param sales_tax_number: A sales tax number is a unique number that identifies a company for tax purposes.
        :param payee_number: A payee number is a unique number that identifies a payee for tax purposes.
        :param abn_or_tfn: An ABN is necessary for operating a business, while a TFN (Tax File Number) is required for any person working in Australia.
        :param abn_branch: An ABN Branch (also known as a GST Branch) is used if part of your business needs to account for GST separately from its parent entity.
        :param acn: The Australian Company Number (ACN) is a nine digit number with the last digit being a check digit calculated using a modified modulus 10 calculation. ASIC has adopted a convention of always printing and displaying the ACN in the format XXX XXX XXX; three blocks of three characters, each block separated by a blank.
        :param first_name: The first name of the person.
        :param last_name: The last name of the person.
        :param bank_accounts:
        :param websites:
        :param addresses:
        :param social_links:
        :param phone_numbers:
        :param emails:
        :param row_type:
        :param custom_fields:
        :param tags:
        :param read_only: Whether the company is read-only or not
        :param salutation: A formal salutation for the person. For example, 'Mr', 'Mrs'
        :param birthday: The date of birth of the person.
        :param pass_through: The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CrmCompaniesUpdateRequest(
            id=id,
            raw=raw,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            company1=models.Company1Input(
                name=name,
                owner_id=owner_id,
                image=image,
                description=description,
                vat_number=vat_number,
                currency=currency,
                status=status,
                fax=fax,
                annual_revenue=annual_revenue,
                number_of_employees=number_of_employees,
                industry=industry,
                ownership=ownership,
                sales_tax_number=sales_tax_number,
                payee_number=payee_number,
                abn_or_tfn=abn_or_tfn,
                abn_branch=abn_branch,
                acn=acn,
                first_name=first_name,
                last_name=last_name,
                bank_accounts=utils.get_pydantic_model(
                    bank_accounts, Optional[List[models.BankAccount1]]
                ),
                websites=utils.get_pydantic_model(
                    websites, Optional[List[models.Website]]
                ),
                addresses=utils.get_pydantic_model(
                    addresses, Optional[List[models.Address]]
                ),
                social_links=utils.get_pydantic_model(
                    social_links, Optional[List[models.SocialLink]]
                ),
                phone_numbers=utils.get_pydantic_model(
                    phone_numbers, Optional[List[models.PhoneNumber]]
                ),
                emails=utils.get_pydantic_model(emails, Optional[List[models.Email]]),
                row_type=utils.get_pydantic_model(
                    row_type, Optional[models.CompanyRowType]
                ),
                custom_fields=utils.get_pydantic_model(
                    custom_fields, Optional[List[models.CustomField]]
                ),
                tags=tags,
                read_only=read_only,
                salutation=salutation,
                birthday=birthday,
                pass_through=utils.get_pydantic_model(
                    pass_through, Optional[List[models.PassThroughBody]]
                ),
            ),
        )

        req = self._build_request(
            method="PATCH",
            path="/crm/companies/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.CrmCompaniesUpdateGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.company1, False, False, "json", models.Company1Input
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="crm.companiesUpdate",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.CrmCompaniesUpdateResponse(
                update_company_response=unmarshal_json_response(
                    Optional[models.UpdateCompanyResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.BadRequestResponseData, http_res
            )
            raise models.BadRequestResponse(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.UnauthorizedResponseData, http_res
            )
            raise models.UnauthorizedResponse(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                models.PaymentRequiredResponseData, http_res
            )
            raise models.PaymentRequiredResponse(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.NotFoundResponseData, http_res
            )
            raise models.NotFoundResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.UnprocessableResponseData, http_res
            )
            raise models.UnprocessableResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "default", "application/json"):
            return models.CrmCompaniesUpdateResponse(
                unexpected_error_response=unmarshal_json_response(
                    Optional[models.UnexpectedErrorResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        raise models.APIError("Unexpected response received", http_res)

    async def update_async(
        self,
        *,
        id: str,
        name: Nullable[str],
        raw: Optional[bool] = False,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        owner_id: OptionalNullable[str] = UNSET,
        image: OptionalNullable[str] = UNSET,
        description: OptionalNullable[str] = UNSET,
        vat_number: OptionalNullable[str] = UNSET,
        currency: OptionalNullable[models.Currency] = UNSET,
        status: OptionalNullable[str] = UNSET,
        fax: OptionalNullable[str] = UNSET,
        annual_revenue: OptionalNullable[str] = UNSET,
        number_of_employees: OptionalNullable[str] = UNSET,
        industry: OptionalNullable[str] = UNSET,
        ownership: OptionalNullable[str] = UNSET,
        sales_tax_number: OptionalNullable[str] = UNSET,
        payee_number: OptionalNullable[str] = UNSET,
        abn_or_tfn: OptionalNullable[str] = UNSET,
        abn_branch: OptionalNullable[str] = UNSET,
        acn: OptionalNullable[str] = UNSET,
        first_name: OptionalNullable[str] = UNSET,
        last_name: OptionalNullable[str] = UNSET,
        bank_accounts: Optional[
            Union[List[models.BankAccount1], List[models.BankAccount1TypedDict]]
        ] = None,
        websites: Optional[
            Union[List[models.Website], List[models.WebsiteTypedDict]]
        ] = None,
        addresses: Optional[
            Union[List[models.Address], List[models.AddressTypedDict]]
        ] = None,
        social_links: Optional[
            Union[List[models.SocialLink], List[models.SocialLinkTypedDict]]
        ] = None,
        phone_numbers: Optional[
            Union[List[models.PhoneNumber], List[models.PhoneNumberTypedDict]]
        ] = None,
        emails: Optional[Union[List[models.Email], List[models.EmailTypedDict]]] = None,
        row_type: Optional[
            Union[models.CompanyRowType, models.CompanyRowTypeTypedDict]
        ] = None,
        custom_fields: Optional[
            Union[List[models.CustomField], List[models.CustomFieldTypedDict]]
        ] = None,
        tags: OptionalNullable[List[str]] = UNSET,
        read_only: OptionalNullable[bool] = UNSET,
        salutation: OptionalNullable[str] = UNSET,
        birthday: OptionalNullable[date] = UNSET,
        pass_through: Optional[
            Union[List[models.PassThroughBody], List[models.PassThroughBodyTypedDict]]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CrmCompaniesUpdateResponse:
        r"""Update company

        Update company

        :param id: ID of the record you are acting upon.
        :param name: Name of the company
        :param raw: Include raw response. Mostly used for debugging purposes
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param owner_id: Owner ID
        :param image: The Image URL of the company
        :param description: A description of the company
        :param vat_number: The VAT number of the company
        :param currency: Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217).
        :param status: The status of the company
        :param fax: The fax number of the company
        :param annual_revenue: The annual revenue of the company
        :param number_of_employees: Number of employees
        :param industry: The industry represents the type of business the company is in.
        :param ownership: The ownership indicates the type of ownership of the company.
        :param sales_tax_number: A sales tax number is a unique number that identifies a company for tax purposes.
        :param payee_number: A payee number is a unique number that identifies a payee for tax purposes.
        :param abn_or_tfn: An ABN is necessary for operating a business, while a TFN (Tax File Number) is required for any person working in Australia.
        :param abn_branch: An ABN Branch (also known as a GST Branch) is used if part of your business needs to account for GST separately from its parent entity.
        :param acn: The Australian Company Number (ACN) is a nine digit number with the last digit being a check digit calculated using a modified modulus 10 calculation. ASIC has adopted a convention of always printing and displaying the ACN in the format XXX XXX XXX; three blocks of three characters, each block separated by a blank.
        :param first_name: The first name of the person.
        :param last_name: The last name of the person.
        :param bank_accounts:
        :param websites:
        :param addresses:
        :param social_links:
        :param phone_numbers:
        :param emails:
        :param row_type:
        :param custom_fields:
        :param tags:
        :param read_only: Whether the company is read-only or not
        :param salutation: A formal salutation for the person. For example, 'Mr', 'Mrs'
        :param birthday: The date of birth of the person.
        :param pass_through: The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CrmCompaniesUpdateRequest(
            id=id,
            raw=raw,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            company1=models.Company1Input(
                name=name,
                owner_id=owner_id,
                image=image,
                description=description,
                vat_number=vat_number,
                currency=currency,
                status=status,
                fax=fax,
                annual_revenue=annual_revenue,
                number_of_employees=number_of_employees,
                industry=industry,
                ownership=ownership,
                sales_tax_number=sales_tax_number,
                payee_number=payee_number,
                abn_or_tfn=abn_or_tfn,
                abn_branch=abn_branch,
                acn=acn,
                first_name=first_name,
                last_name=last_name,
                bank_accounts=utils.get_pydantic_model(
                    bank_accounts, Optional[List[models.BankAccount1]]
                ),
                websites=utils.get_pydantic_model(
                    websites, Optional[List[models.Website]]
                ),
                addresses=utils.get_pydantic_model(
                    addresses, Optional[List[models.Address]]
                ),
                social_links=utils.get_pydantic_model(
                    social_links, Optional[List[models.SocialLink]]
                ),
                phone_numbers=utils.get_pydantic_model(
                    phone_numbers, Optional[List[models.PhoneNumber]]
                ),
                emails=utils.get_pydantic_model(emails, Optional[List[models.Email]]),
                row_type=utils.get_pydantic_model(
                    row_type, Optional[models.CompanyRowType]
                ),
                custom_fields=utils.get_pydantic_model(
                    custom_fields, Optional[List[models.CustomField]]
                ),
                tags=tags,
                read_only=read_only,
                salutation=salutation,
                birthday=birthday,
                pass_through=utils.get_pydantic_model(
                    pass_through, Optional[List[models.PassThroughBody]]
                ),
            ),
        )

        req = self._build_request_async(
            method="PATCH",
            path="/crm/companies/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.CrmCompaniesUpdateGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.company1, False, False, "json", models.Company1Input
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="crm.companiesUpdate",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.CrmCompaniesUpdateResponse(
                update_company_response=unmarshal_json_response(
                    Optional[models.UpdateCompanyResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.BadRequestResponseData, http_res
            )
            raise models.BadRequestResponse(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.UnauthorizedResponseData, http_res
            )
            raise models.UnauthorizedResponse(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                models.PaymentRequiredResponseData, http_res
            )
            raise models.PaymentRequiredResponse(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.NotFoundResponseData, http_res
            )
            raise models.NotFoundResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.UnprocessableResponseData, http_res
            )
            raise models.UnprocessableResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "default", "application/json"):
            return models.CrmCompaniesUpdateResponse(
                unexpected_error_response=unmarshal_json_response(
                    Optional[models.UnexpectedErrorResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        raise models.APIError("Unexpected response received", http_res)

    def delete(
        self,
        *,
        id: str,
        raw: Optional[bool] = False,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CrmCompaniesDeleteResponse:
        r"""Delete company

        Delete company

        :param id: ID of the record you are acting upon.
        :param raw: Include raw response. Mostly used for debugging purposes
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CrmCompaniesDeleteRequest(
            id=id,
            raw=raw,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
        )

        req = self._build_request(
            method="DELETE",
            path="/crm/companies/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.CrmCompaniesDeleteGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="crm.companiesDelete",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.CrmCompaniesDeleteResponse(
                delete_company_response=unmarshal_json_response(
                    Optional[models.DeleteCompanyResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.BadRequestResponseData, http_res
            )
            raise models.BadRequestResponse(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.UnauthorizedResponseData, http_res
            )
            raise models.UnauthorizedResponse(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                models.PaymentRequiredResponseData, http_res
            )
            raise models.PaymentRequiredResponse(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.NotFoundResponseData, http_res
            )
            raise models.NotFoundResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.UnprocessableResponseData, http_res
            )
            raise models.UnprocessableResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "default", "application/json"):
            return models.CrmCompaniesDeleteResponse(
                unexpected_error_response=unmarshal_json_response(
                    Optional[models.UnexpectedErrorResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        raise models.APIError("Unexpected response received", http_res)

    async def delete_async(
        self,
        *,
        id: str,
        raw: Optional[bool] = False,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.CrmCompaniesDeleteResponse:
        r"""Delete company

        Delete company

        :param id: ID of the record you are acting upon.
        :param raw: Include raw response. Mostly used for debugging purposes
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CrmCompaniesDeleteRequest(
            id=id,
            raw=raw,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
        )

        req = self._build_request_async(
            method="DELETE",
            path="/crm/companies/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.CrmCompaniesDeleteGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="crm.companiesDelete",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.CrmCompaniesDeleteResponse(
                delete_company_response=unmarshal_json_response(
                    Optional[models.DeleteCompanyResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.BadRequestResponseData, http_res
            )
            raise models.BadRequestResponse(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.UnauthorizedResponseData, http_res
            )
            raise models.UnauthorizedResponse(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                models.PaymentRequiredResponseData, http_res
            )
            raise models.PaymentRequiredResponse(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.NotFoundResponseData, http_res
            )
            raise models.NotFoundResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.UnprocessableResponseData, http_res
            )
            raise models.UnprocessableResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "default", "application/json"):
            return models.CrmCompaniesDeleteResponse(
                unexpected_error_response=unmarshal_json_response(
                    Optional[models.UnexpectedErrorResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        raise models.APIError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from pydantic import model_serializer
from typing_extensions import NotRequired, TypedDict


class PayrollTotalsTypedDict(TypedDict):
    r"""The overview of the payroll totals."""

    company_debit: NotRequired[Nullable[float]]
    r"""The total company debit for the payroll."""
    tax_debit: NotRequired[Nullable[float]]
    r"""The total tax debit for the payroll."""
    check_amount: NotRequired[Nullable[float]]
    r"""The total check amount for the payroll."""
    net_pay: NotRequired[Nullable[float]]
    r"""The net pay amount for the payroll."""
    gross_pay: NotRequired[Nullable[float]]
    r"""The gross pay amount for the payroll."""
    employer_taxes: NotRequired[Nullable[float]]
    r"""The total amount of employer paid taxes for the payroll."""
    employee_taxes: NotRequired[Nullable[float]]
    r"""The total amount of employee paid taxes for the payroll."""
    employer_benefit_contributions: NotRequired[Nullable[float]]
    r"""The total amount of company contributed benefits for the payroll."""
    employee_benefit_deductions: NotRequired[Nullable[float]]
    r"""The total amount of employee deducted benefits for the payroll."""


class PayrollTotals(BaseModel):
    r"""The overview of the payroll totals."""

    company_debit: OptionalNullable[float] = UNSET
    r"""The total company debit for the payroll."""

    tax_debit: OptionalNullable[float] = UNSET
    r"""The total tax debit for the payroll."""

    check_amount: OptionalNullable[float] = UNSET
    r"""The total check amount for the payroll."""

    net_pay: OptionalNullable[float] = UNSET
    r"""The net pay amount for the payroll."""

    gross_pay: OptionalNullable[float] = UNSET
    r"""The gross pay amount for the payroll."""

    employer_taxes: OptionalNullable[float] = UNSET
    r"""The total amount of employer paid taxes for the payroll."""

    employee_taxes: OptionalNullable[float] = UNSET
    r"""The total amount of employee paid taxes for the payroll."""

    employer_benefit_contributions: OptionalNullable[float] = UNSET
    r"""The total amount of company contributed benefits for the payroll."""

    employee_benefit_deductions: OptionalNullable[float] = UNSET
    r"""The total amount of employee deducted benefits for the payroll."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "company_debit",
            "tax_debit",
            "check_amount",
            "net_pay",
            "gross_pay",
            "employer_taxes",
            "employee_taxes",
            "employer_benefit_contributions",
            "employee_benefit_deductions",
        ]
        nullable_fields = [
            "company_debit",
            "tax_debit",
            "check_amount",
            "net_pay",
            "gross_pay",
            "employer_taxes",
            "employee_taxes",
            "employer_benefit_contributions",
            "employee_benefit_deductions",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .deprecatedlinkedtrackingcategory import (
    DeprecatedLinkedTrackingCategory,
    DeprecatedLinkedTrackingCategoryTypedDict,
)
from .linkedcustomer import LinkedCustomer, LinkedCustomerTypedDict
from .linkedcustomer_input import LinkedCustomerInput, LinkedCustomerInputTypedDict
from .linkedledgeraccount import LinkedLedgerAccount, LinkedLedgerAccountTypedDict
from .linkedledgeraccount_input import (
    LinkedLedgerAccountInput,
    LinkedLedgerAccountInputTypedDict,
)
from .linkedsupplier import LinkedSupplier, LinkedSupplierTypedDict
from .linkedsupplier_input import LinkedSupplierInput, LinkedSupplierInputTypedDict
from .linkedtaxrate import LinkedTaxRate, LinkedTaxRateTypedDict
from .linkedtaxrate_input import LinkedTaxRateInput, LinkedTaxRateInputTypedDict
from .linkedtrackingcategory import (
    LinkedTrackingCategory,
    LinkedTrackingCategoryTypedDict,
)
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from enum import Enum
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class JournalEntryLineItemType(str, Enum):
    r"""Debit entries are considered positive, and credit entries are considered negative."""

    DEBIT = "debit"
    CREDIT = "credit"


class JournalEntryLineItemTypedDict(TypedDict):
    type: JournalEntryLineItemType
    r"""Debit entries are considered positive, and credit entries are considered negative."""
    ledger_account: Nullable[LinkedLedgerAccountTypedDict]
    id: NotRequired[str]
    r"""A unique identifier for an object."""
    description: NotRequired[Nullable[str]]
    r"""User defined description"""
    tax_amount: NotRequired[Nullable[float]]
    r"""Tax amount"""
    sub_total: NotRequired[Nullable[float]]
    r"""Sub-total amount, normally before tax."""
    total_amount: NotRequired[Nullable[float]]
    r"""Debit entries are considered positive, and credit entries are considered negative."""
    tax_rate: NotRequired[LinkedTaxRateTypedDict]
    tracking_category: NotRequired[Nullable[DeprecatedLinkedTrackingCategoryTypedDict]]
    tracking_categories: NotRequired[
        Nullable[List[Nullable[LinkedTrackingCategoryTypedDict]]]
    ]
    r"""A list of linked tracking categories."""
    customer: NotRequired[Nullable[LinkedCustomerTypedDict]]
    r"""The customer this entity is linked to."""
    supplier: NotRequired[Nullable[LinkedSupplierTypedDict]]
    r"""The supplier this entity is linked to."""
    department_id: NotRequired[Nullable[str]]
    r"""The ID of the department"""
    location_id: NotRequired[Nullable[str]]
    r"""The ID of the location"""
    line_number: NotRequired[Nullable[int]]
    r"""Line number of the resource"""


class JournalEntryLineItem(BaseModel):
    type: JournalEntryLineItemType
    r"""Debit entries are considered positive, and credit entries are considered negative."""

    ledger_account: Nullable[LinkedLedgerAccount]

    id: Optional[str] = None
    r"""A unique identifier for an object."""

    description: OptionalNullable[str] = UNSET
    r"""User defined description"""

    tax_amount: OptionalNullable[float] = UNSET
    r"""Tax amount"""

    sub_total: OptionalNullable[float] = UNSET
    r"""Sub-total amount, normally before tax."""

    total_amount: OptionalNullable[float] = UNSET
    r"""Debit entries are considered positive, and credit entries are considered negative."""

    tax_rate: Optional[LinkedTaxRate] = None

    tracking_category: Annotated[
        OptionalNullable[DeprecatedLinkedTrackingCategory],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ] = UNSET

    tracking_categories: OptionalNullable[List[Nullable[LinkedTrackingCategory]]] = (
        UNSET
    )
    r"""A list of linked tracking categories."""

    customer: OptionalNullable[LinkedCustomer] = UNSET
    r"""The customer this entity is linked to."""

    supplier: OptionalNullable[LinkedSupplier] = UNSET
    r"""The supplier this entity is linked to."""

    department_id: OptionalNullable[str] = UNSET
    r"""The ID of the department"""

    location_id: OptionalNullable[str] = UNSET
    r"""The ID of the location"""

    line_number: OptionalNullable[int] = UNSET
    r"""Line number of the resource"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "id",
            "description",
            "tax_amount",
            "sub_total",
            "total_amount",
            "tax_rate",
            "tracking_category",
            "tracking_categories",
            "customer",
            "supplier",
            "department_id",
            "location_id",
            "line_number",
        ]
        nullable_fields = [
            "description",
            "tax_amount",
            "sub_total",
            "total_amount",
            "tracking_category",
            "tracking_categories",
            "ledger_account",
            "customer",
            "supplier",
            "department_id",
            "location_id",
            "line_number",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class JournalEntryLineItemInputTypedDict(TypedDict):
    type: JournalEntryLineItemType
    r"""Debit entries are considered positive, and credit entries are considered negative."""
    ledger_account: Nullable[LinkedLedgerAccountInputTypedDict]
    description: NotRequired[Nullable[str]]
    r"""User defined description"""
    tax_amount: NotRequired[Nullable[float]]
    r"""Tax amount"""
    sub_total: NotRequired[Nullable[float]]
    r"""Sub-total amount, normally before tax."""
    total_amount: NotRequired[Nullable[float]]
    r"""Debit entries are considered positive, and credit entries are considered negative."""
    tax_rate: NotRequired[LinkedTaxRateInputTypedDict]
    tracking_category: NotRequired[Nullable[DeprecatedLinkedTrackingCategoryTypedDict]]
    tracking_categories: NotRequired[
        Nullable[List[Nullable[LinkedTrackingCategoryTypedDict]]]
    ]
    r"""A list of linked tracking categories."""
    customer: NotRequired[Nullable[LinkedCustomerInputTypedDict]]
    r"""The customer this entity is linked to."""
    supplier: NotRequired[Nullable[LinkedSupplierInputTypedDict]]
    r"""The supplier this entity is linked to."""
    department_id: NotRequired[Nullable[str]]
    r"""The ID of the department"""
    location_id: NotRequired[Nullable[str]]
    r"""The ID of the location"""
    line_number: NotRequired[Nullable[int]]
    r"""Line number of the resource"""


class JournalEntryLineItemInput(BaseModel):
    type: JournalEntryLineItemType
    r"""Debit entries are considered positive, and credit entries are considered negative."""

    ledger_account: Nullable[LinkedLedgerAccountInput]

    description: OptionalNullable[str] = UNSET
    r"""User defined description"""

    tax_amount: OptionalNullable[float] = UNSET
    r"""Tax amount"""

    sub_total: OptionalNullable[float] = UNSET
    r"""Sub-total amount, normally before tax."""

    total_amount: OptionalNullable[float] = UNSET
    r"""Debit entries are considered positive, and credit entries are considered negative."""

    tax_rate: Optional[LinkedTaxRateInput] = None

    tracking_category: Annotated[
        OptionalNullable[DeprecatedLinkedTrackingCategory],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ] = UNSET

    tracking_categories: OptionalNullable[List[Nullable[LinkedTrackingCategory]]] = (
        UNSET
    )
    r"""A list of linked tracking categories."""

    customer: OptionalNullable[LinkedCustomerInput] = UNSET
    r"""The customer this entity is linked to."""

    supplier: OptionalNullable[LinkedSupplierInput] = UNSET
    r"""The supplier this entity is linked to."""

    department_id: OptionalNullable[str] = UNSET
    r"""The ID of the department"""

    location_id: OptionalNullable[str] = UNSET
    r"""The ID of the location"""

    line_number: OptionalNullable[int] = UNSET
    r"""Line number of the resource"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "description",
            "tax_amount",
            "sub_total",
            "total_amount",
            "tax_rate",
            "tracking_category",
            "tracking_categories",
            "customer",
            "supplier",
            "department_id",
            "location_id",
            "line_number",
        ]
        nullable_fields = [
            "description",
            "tax_amount",
            "sub_total",
            "total_amount",
            "tracking_category",
            "tracking_categories",
            "ledger_account",
            "customer",
            "supplier",
            "department_id",
            "location_id",
            "line_number",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .currency import Currency
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from datetime import datetime
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class EcommerceOrderRefundTypedDict(TypedDict):
    r"""A refund for an ecommerce order."""

    id: NotRequired[Nullable[str]]
    r"""A unique identifier for an object."""
    amount: NotRequired[str]
    r"""The amount of the refund."""
    currency: NotRequired[Nullable[Currency]]
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""
    reason: NotRequired[str]
    r"""The reason for the refund."""
    created_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was created."""


class EcommerceOrderRefund(BaseModel):
    r"""A refund for an ecommerce order."""

    id: OptionalNullable[str] = UNSET
    r"""A unique identifier for an object."""

    amount: Optional[str] = None
    r"""The amount of the refund."""

    currency: OptionalNullable[Currency] = UNSET
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""

    reason: Optional[str] = None
    r"""The reason for the refund."""

    created_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was created."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["id", "amount", "currency", "reason", "created_at"]
        nullable_fields = ["id", "currency", "created_at"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from enum import Enum
from pydantic import model_serializer
from typing_extensions import NotRequired, TypedDict


class OperationTypedDict(TypedDict):
    r"""The request as defined in OpenApi Spec."""

    id: str
    r"""The OpenApi Operation Id associated with the request"""
    name: str
    r"""The OpenApi Operation name associated with the request"""


class Operation(BaseModel):
    r"""The request as defined in OpenApi Spec."""

    id: str
    r"""The OpenApi Operation Id associated with the request"""

    name: str
    r"""The OpenApi Operation name associated with the request"""


class ServiceTypedDict(TypedDict):
    r"""Apideck service provider associated with request."""

    id: str
    r"""Apideck service provider id."""
    name: str
    r"""Apideck service provider name."""


class Service(BaseModel):
    r"""Apideck service provider associated with request."""

    id: str
    r"""Apideck service provider id."""

    name: str
    r"""Apideck service provider name."""


class UnifiedAPI(str, Enum):
    r"""Which Unified Api request was made to."""

    CRM = "crm"
    LEAD = "lead"
    PROXY = "proxy"
    VAULT = "vault"
    ACCOUNTING = "accounting"
    HRIS = "hris"
    ATS = "ats"
    ECOMMERCE = "ecommerce"
    ISSUE_TRACKING = "issue-tracking"
    POS = "pos"
    FILE_STORAGE = "file-storage"
    SMS = "sms"


class LogTypedDict(TypedDict):
    api_style: str
    r"""Indicates if the request was made via REST or Graphql endpoint."""
    base_url: str
    r"""The Apideck base URL the request was made to."""
    child_request: bool
    r"""Indicates whether or not this is a child or parent request."""
    consumer_id: str
    r"""The consumer Id associated with the request."""
    duration: float
    r"""The entire execution time in milliseconds it took to call the Apideck service provider."""
    execution: int
    r"""The entire execution time in milliseconds it took to make the request."""
    has_children: bool
    r"""When request is a parent request, this indicates if there are child requests associated."""
    http_method: str
    r"""HTTP Method of request."""
    id: str
    r"""UUID acting as Request Identifier."""
    latency: float
    r"""Latency added by making this request via Unified Api."""
    operation: OperationTypedDict
    r"""The request as defined in OpenApi Spec."""
    parent_id: Nullable[str]
    r"""When request is a child request, this UUID indicates it's parent request."""
    path: str
    r"""The path component of the URI the request was made to."""
    sandbox: bool
    r"""Indicates whether the request was made using Apidecks sandbox credentials or not."""
    service: ServiceTypedDict
    r"""Apideck service provider associated with request."""
    status_code: int
    r"""HTTP Status code that was returned."""
    success: bool
    r"""Whether or not the request was successful."""
    timestamp: str
    r"""ISO Date and time when the request was made."""
    unified_api: UnifiedAPI
    r"""Which Unified Api request was made to."""
    error_message: NotRequired[Nullable[str]]
    r"""If error occurred, this is brief explanation"""
    source_ip: NotRequired[Nullable[str]]
    r"""The IP address of the source of the request."""


class Log(BaseModel):
    api_style: str
    r"""Indicates if the request was made via REST or Graphql endpoint."""

    base_url: str
    r"""The Apideck base URL the request was made to."""

    child_request: bool
    r"""Indicates whether or not this is a child or parent request."""

    consumer_id: str
    r"""The consumer Id associated with the request."""

    duration: float
    r"""The entire execution time in milliseconds it took to call the Apideck service provider."""

    execution: int
    r"""The entire execution time in milliseconds it took to make the request."""

    has_children: bool
    r"""When request is a parent request, this indicates if there are child requests associated."""

    http_method: str
    r"""HTTP Method of request."""

    id: str
    r"""UUID acting as Request Identifier."""

    latency: float
    r"""Latency added by making this request via Unified Api."""

    operation: Operation
    r"""The request as defined in OpenApi Spec."""

    parent_id: Nullable[str]
    r"""When request is a child request, this UUID indicates it's parent request."""

    path: str
    r"""The path component of the URI the request was made to."""

    sandbox: bool
    r"""Indicates whether the request was made using Apidecks sandbox credentials or not."""

    service: Service
    r"""Apideck service provider associated with request."""

    status_code: int
    r"""HTTP Status code that was returned."""

    success: bool
    r"""Whether or not the request was successful."""

    timestamp: str
    r"""ISO Date and time when the request was made."""

    unified_api: UnifiedAPI
    r"""Which Unified Api request was made to."""

    error_message: OptionalNullable[str] = UNSET
    r"""If error occurred, this is brief explanation"""

    source_ip: OptionalNullable[str] = UNSET
    r"""The IP address of the source of the request."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["error_message", "source_ip"]
        nullable_fields = ["error_message", "parent_id", "source_ip"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ConnectorStatus(str, Enum):
    r"""Status of the connector. Connectors with status live or beta are callable."""

    LIVE = "live"
    BETA = "beta"
    EARLY_ACCESS = "early-access"
    DEVELOPMENT = "development"
    CONSIDERING = "considering"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .address import Address, AddressTypedDict
from .email import Email, EmailTypedDict
from .passthroughbody import PassThroughBody, PassThroughBodyTypedDict
from .phonenumber import PhoneNumber, PhoneNumberTypedDict
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
import pydantic
from pydantic import model_serializer
from typing import Any, Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class UserTypedDict(TypedDict):
    emails: List[EmailTypedDict]
    id: NotRequired[str]
    r"""The unique identifier for the user"""
    parent_id: NotRequired[Nullable[str]]
    r"""The parent user id"""
    username: NotRequired[Nullable[str]]
    r"""The username of the user"""
    first_name: NotRequired[Nullable[str]]
    r"""The first name of the person."""
    last_name: NotRequired[Nullable[str]]
    r"""The last name of the person."""
    title: NotRequired[Nullable[str]]
    r"""The job title of the person."""
    division: NotRequired[Nullable[str]]
    r"""The division the person is currently in. Usually a collection of departments or teams or regions."""
    department: NotRequired[Nullable[str]]
    r"""The department the person is currently in. [Deprecated](https://developers.apideck.com/changelog) in favor of the dedicated department_id and department_name field."""
    company_name: NotRequired[Nullable[str]]
    r"""The name of the company."""
    employee_number: NotRequired[Nullable[str]]
    r"""An Employee Number, Employee ID or Employee Code, is a unique number that has been assigned to each individual staff member within a company."""
    description: NotRequired[Nullable[str]]
    r"""A description of the object."""
    image: NotRequired[Nullable[str]]
    r"""The URL of the user's avatar"""
    language: NotRequired[Nullable[str]]
    r"""language code according to ISO 639-1. For the United States - EN"""
    status: NotRequired[Nullable[str]]
    r"""The status of the user"""
    addresses: NotRequired[List[AddressTypedDict]]
    phone_numbers: NotRequired[List[PhoneNumberTypedDict]]
    custom_mappings: NotRequired[Nullable[Dict[str, Any]]]
    r"""When custom mappings are configured on the resource, the result is included here."""
    updated_at: NotRequired[Nullable[str]]
    r"""The date and time when the user was last updated."""
    created_at: NotRequired[Nullable[str]]
    r"""The date and time when the user was created."""
    pass_through: NotRequired[List[PassThroughBodyTypedDict]]
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""


class User(BaseModel):
    emails: List[Email]

    id: Optional[str] = None
    r"""The unique identifier for the user"""

    parent_id: OptionalNullable[str] = UNSET
    r"""The parent user id"""

    username: OptionalNullable[str] = UNSET
    r"""The username of the user"""

    first_name: OptionalNullable[str] = UNSET
    r"""The first name of the person."""

    last_name: OptionalNullable[str] = UNSET
    r"""The last name of the person."""

    title: OptionalNullable[str] = UNSET
    r"""The job title of the person."""

    division: OptionalNullable[str] = UNSET
    r"""The division the person is currently in. Usually a collection of departments or teams or regions."""

    department: Annotated[
        OptionalNullable[str],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ] = UNSET
    r"""The department the person is currently in. [Deprecated](https://developers.apideck.com/changelog) in favor of the dedicated department_id and department_name field."""

    company_name: OptionalNullable[str] = UNSET
    r"""The name of the company."""

    employee_number: OptionalNullable[str] = UNSET
    r"""An Employee Number, Employee ID or Employee Code, is a unique number that has been assigned to each individual staff member within a company."""

    description: OptionalNullable[str] = UNSET
    r"""A description of the object."""

    image: OptionalNullable[str] = UNSET
    r"""The URL of the user's avatar"""

    language: OptionalNullable[str] = UNSET
    r"""language code according to ISO 639-1. For the United States - EN"""

    status: OptionalNullable[str] = UNSET
    r"""The status of the user"""

    addresses: Optional[List[Address]] = None

    phone_numbers: Optional[List[PhoneNumber]] = None

    custom_mappings: OptionalNullable[Dict[str, Any]] = UNSET
    r"""When custom mappings are configured on the resource, the result is included here."""

    updated_at: OptionalNullable[str] = UNSET
    r"""The date and time when the user was last updated."""

    created_at: OptionalNullable[str] = UNSET
    r"""The date and time when the user was created."""

    pass_through: Optional[List[PassThroughBody]] = None
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "id",
            "parent_id",
            "username",
            "first_name",
            "last_name",
            "title",
            "division",
            "department",
            "company_name",
            "employee_number",
            "description",
            "image",
            "language",
            "status",
            "addresses",
            "phone_numbers",
            "custom_mappings",
            "updated_at",
            "created_at",
            "pass_through",
        ]
        nullable_fields = [
            "parent_id",
            "username",
            "first_name",
            "last_name",
            "title",
            "division",
            "department",
            "company_name",
            "employee_number",
            "description",
            "image",
            "language",
            "status",
            "custom_mappings",
            "updated_at",
            "created_at",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .passthroughbody import PassThroughBody, PassThroughBodyTypedDict
from apideck_unify.types import BaseModel, Nullable, UNSET_SENTINEL
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class CreateUploadSessionRequestTypedDict(TypedDict):
    name: str
    r"""The name of the file."""
    parent_folder_id: str
    r"""The parent folder to create the new file within. This can be an ID or a path depending on the downstream folder. Please see the connector section below to see downstream specific gotchas."""
    size: Nullable[int]
    r"""The size of the file in bytes"""
    drive_id: NotRequired[str]
    r"""ID of the drive to upload to."""
    pass_through: NotRequired[List[PassThroughBodyTypedDict]]
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""


class CreateUploadSessionRequest(BaseModel):
    name: str
    r"""The name of the file."""

    parent_folder_id: str
    r"""The parent folder to create the new file within. This can be an ID or a path depending on the downstream folder. Please see the connector section below to see downstream specific gotchas."""

    size: Nullable[int]
    r"""The size of the file in bytes"""

    drive_id: Optional[str] = None
    r"""ID of the drive to upload to."""

    pass_through: Optional[List[PassThroughBody]] = None
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["drive_id", "pass_through"]
        nullable_fields = ["size"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .getconnectionsresponse import (
    GetConnectionsResponse,
    GetConnectionsResponseTypedDict,
)
from .httpmetadata import HTTPMetadata, HTTPMetadataTypedDict
from .unexpectederrorresponse import (
    UnexpectedErrorResponse,
    UnexpectedErrorResponseTypedDict,
)
from apideck_unify.types import BaseModel
from apideck_unify.utils import FieldMetadata, HeaderMetadata, QueryParamMetadata
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class VaultConnectionsAllGlobalsTypedDict(TypedDict):
    consumer_id: NotRequired[str]
    r"""ID of the consumer which you want to get or push data from"""
    app_id: NotRequired[str]
    r"""The ID of your Unify application"""


class VaultConnectionsAllGlobals(BaseModel):
    consumer_id: Annotated[
        Optional[str],
        pydantic.Field(alias="x-apideck-consumer-id"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""ID of the consumer which you want to get or push data from"""

    app_id: Annotated[
        Optional[str],
        pydantic.Field(alias="x-apideck-app-id"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The ID of your Unify application"""


class VaultConnectionsAllRequestTypedDict(TypedDict):
    consumer_id: NotRequired[str]
    r"""ID of the consumer which you want to get or push data from"""
    app_id: NotRequired[str]
    r"""The ID of your Unify application"""
    api: NotRequired[str]
    r"""Scope results to Unified API"""
    configured: NotRequired[bool]
    r"""Scopes results to connections that have been configured or not"""


class VaultConnectionsAllRequest(BaseModel):
    consumer_id: Annotated[
        Optional[str],
        pydantic.Field(alias="x-apideck-consumer-id"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""ID of the consumer which you want to get or push data from"""

    app_id: Annotated[
        Optional[str],
        pydantic.Field(alias="x-apideck-app-id"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The ID of your Unify application"""

    api: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Scope results to Unified API"""

    configured: Annotated[
        Optional[bool],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Scopes results to connections that have been configured or not"""


class VaultConnectionsAllResponseTypedDict(TypedDict):
    http_meta: HTTPMetadataTypedDict
    get_connections_response: NotRequired[GetConnectionsResponseTypedDict]
    r"""Connections"""
    unexpected_error_response: NotRequired[UnexpectedErrorResponseTypedDict]
    r"""Unexpected error"""


class VaultConnectionsAllResponse(BaseModel):
    http_meta: Annotated[Optional[HTTPMetadata], pydantic.Field(exclude=True)] = None

    get_connections_response: Optional[GetConnectionsResponse] = None
    r"""Connections"""

    unexpected_error_response: Optional[UnexpectedErrorResponse] = None
    r"""Unexpected error"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import BaseModel
from enum import Enum
from typing import Dict, List, Optional
from typing_extensions import NotRequired, TypedDict


class Mode(str, Enum):
    r"""Mode of the webhook support."""

    NATIVE = "native"
    VIRTUAL = "virtual"
    NONE = "none"


class SubscriptionLevel(str, Enum):
    r"""Received events are scoped to connection or across integration."""

    CONNECTION = "connection"
    INTEGRATION = "integration"


class ManagedVia(str, Enum):
    r"""How the subscription is managed in the downstream."""

    MANUAL = "manual"
    API = "api"


class Unit(str, Enum):
    r"""The window unit for the rate."""

    SECOND = "second"
    MINUTE = "minute"
    HOUR = "hour"
    DAY = "day"


class RequestRateTypedDict(TypedDict):
    r"""The rate at which requests for resources will be made to downstream."""

    rate: int
    r"""The number of requests per window unit."""
    size: int
    r"""Size of request window."""
    unit: Unit
    r"""The window unit for the rate."""


class RequestRate(BaseModel):
    r"""The rate at which requests for resources will be made to downstream."""

    rate: int
    r"""The number of requests per window unit."""

    size: int
    r"""Size of request window."""

    unit: Unit
    r"""The window unit for the rate."""


class WebhookSupportResourcesTypedDict(TypedDict):
    events: NotRequired[List[str]]


class WebhookSupportResources(BaseModel):
    events: Optional[List[str]] = None


class VirtualWebhooksTypedDict(TypedDict):
    r"""Virtual webhook config for the connector."""

    request_rate: RequestRateTypedDict
    r"""The rate at which requests for resources will be made to downstream."""
    resources: NotRequired[Dict[str, WebhookSupportResourcesTypedDict]]
    r"""The resources that will be requested from downstream."""


class VirtualWebhooks(BaseModel):
    r"""Virtual webhook config for the connector."""

    request_rate: RequestRate
    r"""The rate at which requests for resources will be made to downstream."""

    resources: Optional[Dict[str, WebhookSupportResources]] = None
    r"""The resources that will be requested from downstream."""


class WebhookSupportTypedDict(TypedDict):
    r"""How webhooks are supported for the connector. Sometimes the connector natively supports webhooks, other times Apideck virtualizes them based on polling."""

    mode: NotRequired[Mode]
    r"""Mode of the webhook support."""
    subscription_level: NotRequired[SubscriptionLevel]
    r"""Received events are scoped to connection or across integration."""
    managed_via: NotRequired[ManagedVia]
    r"""How the subscription is managed in the downstream."""
    virtual_webhooks: NotRequired[VirtualWebhooksTypedDict]
    r"""Virtual webhook config for the connector."""


class WebhookSupport(BaseModel):
    r"""How webhooks are supported for the connector. Sometimes the connector natively supports webhooks, other times Apideck virtualizes them based on polling."""

    mode: Optional[Mode] = None
    r"""Mode of the webhook support."""

    subscription_level: Optional[SubscriptionLevel] = None
    r"""Received events are scoped to connection or across integration."""

    managed_via: Optional[ManagedVia] = None
    r"""How the subscription is managed in the downstream."""

    virtual_webhooks: Optional[VirtualWebhooks] = None
    r"""Virtual webhook config for the connector."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .currency import Currency
from .customfield import CustomField, CustomFieldTypedDict
from .expenselineitem import ExpenseLineItem, ExpenseLineItemTypedDict
from .expenselineitem_input import ExpenseLineItemInput, ExpenseLineItemInputTypedDict
from .linkedtaxrate import LinkedTaxRate, LinkedTaxRateTypedDict
from .linkedtaxrate_input import LinkedTaxRateInput, LinkedTaxRateInputTypedDict
from .passthroughbody import PassThroughBody, PassThroughBodyTypedDict
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from datetime import datetime
from enum import Enum
from pydantic import model_serializer
from typing import Any, Dict, List, Optional
from typing_extensions import NotRequired, TypedDict


class ExpensePaymentType(str, Enum):
    r"""The type of payment for the expense."""

    CASH = "cash"
    CHECK = "check"
    CREDIT_CARD = "credit_card"


class ExpenseType(str, Enum):
    r"""The type of expense."""

    EXPENSE = "expense"
    REFUND = "refund"


class ExpenseTypedDict(TypedDict):
    transaction_date: Nullable[datetime]
    r"""The date of the transaction - YYYY:MM::DDThh:mm:ss.sTZD"""
    account_id: str
    r"""The unique identifier for the ledger account that this expense should be credited to."""
    line_items: List[ExpenseLineItemTypedDict]
    r"""Expense line items linked to this expense."""
    id: NotRequired[str]
    r"""A unique identifier for an object."""
    number: NotRequired[Nullable[str]]
    r"""Number."""
    customer_id: NotRequired[str]
    r"""The ID of the customer this entity is linked to. Used for expenses that should be marked as billable to customers."""
    supplier_id: NotRequired[str]
    r"""The ID of the supplier this entity is linked to."""
    company_id: NotRequired[Nullable[str]]
    r"""The company ID the transaction belongs to"""
    department_id: NotRequired[Nullable[str]]
    r"""The ID of the department"""
    payment_type: NotRequired[Nullable[ExpensePaymentType]]
    r"""The type of payment for the expense."""
    currency: NotRequired[Nullable[Currency]]
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""
    currency_rate: NotRequired[Nullable[float]]
    r"""Currency Exchange Rate at the time entity was recorded/generated."""
    type: NotRequired[Nullable[ExpenseType]]
    r"""The type of expense."""
    memo: NotRequired[Nullable[str]]
    r"""The memo of the expense."""
    tax_rate: NotRequired[LinkedTaxRateTypedDict]
    total_amount: NotRequired[Nullable[float]]
    r"""The total amount of the expense line item."""
    reference: NotRequired[Nullable[str]]
    r"""Optional reference identifier for the transaction."""
    source_document_url: NotRequired[Nullable[str]]
    r"""URL link to a source document - shown as 'Go to [appName]' in the downstream app. Currently only supported for Xero."""
    custom_fields: NotRequired[List[CustomFieldTypedDict]]
    custom_mappings: NotRequired[Nullable[Dict[str, Any]]]
    r"""When custom mappings are configured on the resource, the result is included here."""
    updated_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was last updated."""
    created_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was created."""
    row_version: NotRequired[Nullable[str]]
    r"""A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object."""
    updated_by: NotRequired[Nullable[str]]
    r"""The user who last updated the object."""
    created_by: NotRequired[Nullable[str]]
    r"""The user who created the object."""
    pass_through: NotRequired[List[PassThroughBodyTypedDict]]
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""


class Expense(BaseModel):
    transaction_date: Nullable[datetime]
    r"""The date of the transaction - YYYY:MM::DDThh:mm:ss.sTZD"""

    account_id: str
    r"""The unique identifier for the ledger account that this expense should be credited to."""

    line_items: List[ExpenseLineItem]
    r"""Expense line items linked to this expense."""

    id: Optional[str] = None
    r"""A unique identifier for an object."""

    number: OptionalNullable[str] = UNSET
    r"""Number."""

    customer_id: Optional[str] = None
    r"""The ID of the customer this entity is linked to. Used for expenses that should be marked as billable to customers."""

    supplier_id: Optional[str] = None
    r"""The ID of the supplier this entity is linked to."""

    company_id: OptionalNullable[str] = UNSET
    r"""The company ID the transaction belongs to"""

    department_id: OptionalNullable[str] = UNSET
    r"""The ID of the department"""

    payment_type: OptionalNullable[ExpensePaymentType] = UNSET
    r"""The type of payment for the expense."""

    currency: OptionalNullable[Currency] = UNSET
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""

    currency_rate: OptionalNullable[float] = UNSET
    r"""Currency Exchange Rate at the time entity was recorded/generated."""

    type: OptionalNullable[ExpenseType] = UNSET
    r"""The type of expense."""

    memo: OptionalNullable[str] = UNSET
    r"""The memo of the expense."""

    tax_rate: Optional[LinkedTaxRate] = None

    total_amount: OptionalNullable[float] = UNSET
    r"""The total amount of the expense line item."""

    reference: OptionalNullable[str] = UNSET
    r"""Optional reference identifier for the transaction."""

    source_document_url: OptionalNullable[str] = UNSET
    r"""URL link to a source document - shown as 'Go to [appName]' in the downstream app. Currently only supported for Xero."""

    custom_fields: Optional[List[CustomField]] = None

    custom_mappings: OptionalNullable[Dict[str, Any]] = UNSET
    r"""When custom mappings are configured on the resource, the result is included here."""

    updated_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was last updated."""

    created_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was created."""

    row_version: OptionalNullable[str] = UNSET
    r"""A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object."""

    updated_by: OptionalNullable[str] = UNSET
    r"""The user who last updated the object."""

    created_by: OptionalNullable[str] = UNSET
    r"""The user who created the object."""

    pass_through: Optional[List[PassThroughBody]] = None
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "id",
            "number",
            "customer_id",
            "supplier_id",
            "company_id",
            "department_id",
            "payment_type",
            "currency",
            "currency_rate",
            "type",
            "memo",
            "tax_rate",
            "total_amount",
            "reference",
            "source_document_url",
            "custom_fields",
            "custom_mappings",
            "updated_at",
            "created_at",
            "row_version",
            "updated_by",
            "created_by",
            "pass_through",
        ]
        nullable_fields = [
            "number",
            "transaction_date",
            "company_id",
            "department_id",
            "payment_type",
            "currency",
            "currency_rate",
            "type",
            "memo",
            "total_amount",
            "reference",
            "source_document_url",
            "custom_mappings",
            "updated_at",
            "created_at",
            "row_version",
            "updated_by",
            "created_by",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class ExpenseInputTypedDict(TypedDict):
    transaction_date: Nullable[datetime]
    r"""The date of the transaction - YYYY:MM::DDThh:mm:ss.sTZD"""
    account_id: str
    r"""The unique identifier for the ledger account that this expense should be credited to."""
    line_items: List[ExpenseLineItemInputTypedDict]
    r"""Expense line items linked to this expense."""
    number: NotRequired[Nullable[str]]
    r"""Number."""
    customer_id: NotRequired[str]
    r"""The ID of the customer this entity is linked to. Used for expenses that should be marked as billable to customers."""
    supplier_id: NotRequired[str]
    r"""The ID of the supplier this entity is linked to."""
    company_id: NotRequired[Nullable[str]]
    r"""The company ID the transaction belongs to"""
    department_id: NotRequired[Nullable[str]]
    r"""The ID of the department"""
    payment_type: NotRequired[Nullable[ExpensePaymentType]]
    r"""The type of payment for the expense."""
    currency: NotRequired[Nullable[Currency]]
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""
    currency_rate: NotRequired[Nullable[float]]
    r"""Currency Exchange Rate at the time entity was recorded/generated."""
    type: NotRequired[Nullable[ExpenseType]]
    r"""The type of expense."""
    memo: NotRequired[Nullable[str]]
    r"""The memo of the expense."""
    tax_rate: NotRequired[LinkedTaxRateInputTypedDict]
    total_amount: NotRequired[Nullable[float]]
    r"""The total amount of the expense line item."""
    reference: NotRequired[Nullable[str]]
    r"""Optional reference identifier for the transaction."""
    source_document_url: NotRequired[Nullable[str]]
    r"""URL link to a source document - shown as 'Go to [appName]' in the downstream app. Currently only supported for Xero."""
    custom_fields: NotRequired[List[CustomFieldTypedDict]]
    row_version: NotRequired[Nullable[str]]
    r"""A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object."""
    pass_through: NotRequired[List[PassThroughBodyTypedDict]]
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""


class ExpenseInput(BaseModel):
    transaction_date: Nullable[datetime]
    r"""The date of the transaction - YYYY:MM::DDThh:mm:ss.sTZD"""

    account_id: str
    r"""The unique identifier for the ledger account that this expense should be credited to."""

    line_items: List[ExpenseLineItemInput]
    r"""Expense line items linked to this expense."""

    number: OptionalNullable[str] = UNSET
    r"""Number."""

    customer_id: Optional[str] = None
    r"""The ID of the customer this entity is linked to. Used for expenses that should be marked as billable to customers."""

    supplier_id: Optional[str] = None
    r"""The ID of the supplier this entity is linked to."""

    company_id: OptionalNullable[str] = UNSET
    r"""The company ID the transaction belongs to"""

    department_id: OptionalNullable[str] = UNSET
    r"""The ID of the department"""

    payment_type: OptionalNullable[ExpensePaymentType] = UNSET
    r"""The type of payment for the expense."""

    currency: OptionalNullable[Currency] = UNSET
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""

    currency_rate: OptionalNullable[float] = UNSET
    r"""Currency Exchange Rate at the time entity was recorded/generated."""

    type: OptionalNullable[ExpenseType] = UNSET
    r"""The type of expense."""

    memo: OptionalNullable[str] = UNSET
    r"""The memo of the expense."""

    tax_rate: Optional[LinkedTaxRateInput] = None

    total_amount: OptionalNullable[float] = UNSET
    r"""The total amount of the expense line item."""

    reference: OptionalNullable[str] = UNSET
    r"""Optional reference identifier for the transaction."""

    source_document_url: OptionalNullable[str] = UNSET
    r"""URL link to a source document - shown as 'Go to [appName]' in the downstream app. Currently only supported for Xero."""

    custom_fields: Optional[List[CustomField]] = None

    row_version: OptionalNullable[str] = UNSET
    r"""A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object."""

    pass_through: Optional[List[PassThroughBody]] = None
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "number",
            "customer_id",
            "supplier_id",
            "company_id",
            "department_id",
            "payment_type",
            "currency",
            "currency_rate",
            "type",
            "memo",
            "tax_rate",
            "total_amount",
            "reference",
            "source_document_url",
            "custom_fields",
            "row_version",
            "pass_through",
        ]
        nullable_fields = [
            "number",
            "transaction_date",
            "company_id",
            "department_id",
            "payment_type",
            "currency",
            "currency_rate",
            "type",
            "memo",
            "total_amount",
            "reference",
            "source_document_url",
            "row_version",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from pydantic import model_serializer
from typing_extensions import NotRequired, TypedDict


class EcommerceAddressTypedDict(TypedDict):
    r"""An object representing a shipping or billing address."""

    line1: NotRequired[Nullable[str]]
    r"""Address line 1 of the billing address."""
    line2: NotRequired[Nullable[str]]
    r"""Address line 2 of the billing address."""
    company_name: NotRequired[Nullable[str]]
    r"""Company name of the customer"""
    city: NotRequired[Nullable[str]]
    r"""City of the billing address."""
    state: NotRequired[Nullable[str]]
    r"""State/province of the billing address."""
    postal_code: NotRequired[Nullable[str]]
    r"""Postal/ZIP code of the billing address."""
    country: NotRequired[Nullable[str]]
    r"""Country of the billing address."""


class EcommerceAddress(BaseModel):
    r"""An object representing a shipping or billing address."""

    line1: OptionalNullable[str] = UNSET
    r"""Address line 1 of the billing address."""

    line2: OptionalNullable[str] = UNSET
    r"""Address line 2 of the billing address."""

    company_name: OptionalNullable[str] = UNSET
    r"""Company name of the customer"""

    city: OptionalNullable[str] = UNSET
    r"""City of the billing address."""

    state: OptionalNullable[str] = UNSET
    r"""State/province of the billing address."""

    postal_code: OptionalNullable[str] = UNSET
    r"""Postal/ZIP code of the billing address."""

    country: OptionalNullable[str] = UNSET
    r"""Country of the billing address."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "line1",
            "line2",
            "company_name",
            "city",
            "state",
            "postal_code",
            "country",
        ]
        nullable_fields = [
            "line1",
            "line2",
            "company_name",
            "city",
            "state",
            "postal_code",
            "country",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import BaseModel
from datetime import date
from enum import Enum
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class TransactionType(str, Enum):
    r"""Type of the transaction."""

    INVOICE = "invoice"
    CREDIT_NOTE = "credit_note"
    BILL = "bill"
    PAYMENT = "payment"
    BILL_PAYMENT = "bill_payment"


class BalanceByTransactionTypedDict(TypedDict):
    transaction_id: NotRequired[str]
    r"""Unique identifier for the transaction."""
    transaction_date: NotRequired[date]
    r"""Date of the transaction."""
    transaction_type: NotRequired[TransactionType]
    r"""Type of the transaction."""
    due_date: NotRequired[date]
    r"""Due date of the transaction."""
    original_amount: NotRequired[float]
    r"""Original amount of the transaction."""
    outstanding_balance: NotRequired[float]
    r"""Outstanding balance of the transaction."""
    transaction_number: NotRequired[str]
    r"""Transaction number of the transaction."""


class BalanceByTransaction(BaseModel):
    transaction_id: Optional[str] = None
    r"""Unique identifier for the transaction."""

    transaction_date: Optional[date] = None
    r"""Date of the transaction."""

    transaction_type: Optional[TransactionType] = None
    r"""Type of the transaction."""

    due_date: Optional[date] = None
    r"""Due date of the transaction."""

    original_amount: Optional[float] = None
    r"""Original amount of the transaction."""

    outstanding_balance: Optional[float] = None
    r"""Outstanding balance of the transaction."""

    transaction_number: Optional[str] = None
    r"""Transaction number of the transaction."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import BaseModel
from enum import Enum
from typing import List, Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


Value5TypedDict = TypeAliasType("Value5TypedDict", Union[str, int, float])


Value5 = TypeAliasType("Value5", Union[str, int, float])


SimpleFormFieldOptionValueTypedDict = TypeAliasType(
    "SimpleFormFieldOptionValueTypedDict",
    Union[str, int, float, bool, List[Value5TypedDict]],
)


SimpleFormFieldOptionValue = TypeAliasType(
    "SimpleFormFieldOptionValue", Union[str, int, float, bool, List[Value5]]
)


class OptionType(str, Enum):
    SIMPLE = "simple"


class SimpleFormFieldOptionTypedDict(TypedDict):
    label: str
    option_type: OptionType
    value: NotRequired[SimpleFormFieldOptionValueTypedDict]


class SimpleFormFieldOption(BaseModel):
    label: str

    option_type: OptionType

    value: Optional[SimpleFormFieldOptionValue] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .currency import Currency
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from datetime import datetime
from pydantic import model_serializer
from typing import Any, Dict, List, Optional
from typing_extensions import NotRequired, TypedDict


class BalanceSheetAssetsAccountTypedDict(TypedDict):
    r"""A balance sheet assets account represents the financial position of a company at a specific point in time."""

    account_id: NotRequired[str]
    r"""The unique identifier for the account."""
    code: NotRequired[str]
    r"""The account code of the account"""
    name: NotRequired[str]
    r"""The name of the account."""
    value: NotRequired[float]
    r"""The amount or value of the item"""
    items: NotRequired[Any]
    r"""A list of balance sheet accounts"""


class BalanceSheetAssetsAccount(BaseModel):
    r"""A balance sheet assets account represents the financial position of a company at a specific point in time."""

    account_id: Optional[str] = None
    r"""The unique identifier for the account."""

    code: Optional[str] = None
    r"""The account code of the account"""

    name: Optional[str] = None
    r"""The name of the account."""

    value: Optional[float] = None
    r"""The amount or value of the item"""

    items: Optional[Any] = None
    r"""A list of balance sheet accounts"""


class BalanceSheetLiabilitiesAccountTypedDict(TypedDict):
    r"""A balance sheet liabilities account represents the financial position of a company at a specific point in time."""

    account_id: NotRequired[str]
    r"""The unique identifier for the account."""
    code: NotRequired[str]
    r"""The account code of the account"""
    name: NotRequired[str]
    r"""The name of the account."""
    value: NotRequired[float]
    r"""The amount or value of the item"""
    items: NotRequired[Any]
    r"""A list of balance sheet accounts"""


class BalanceSheetLiabilitiesAccount(BaseModel):
    r"""A balance sheet liabilities account represents the financial position of a company at a specific point in time."""

    account_id: Optional[str] = None
    r"""The unique identifier for the account."""

    code: Optional[str] = None
    r"""The account code of the account"""

    name: Optional[str] = None
    r"""The name of the account."""

    value: Optional[float] = None
    r"""The amount or value of the item"""

    items: Optional[Any] = None
    r"""A list of balance sheet accounts"""


class BalanceSheetEquityAccountTypedDict(TypedDict):
    r"""A balance sheet equity account represents the financial position of a company at a specific point in time."""

    account_id: NotRequired[str]
    r"""The unique identifier for the account."""
    code: NotRequired[str]
    r"""The account code of the account"""
    name: NotRequired[str]
    r"""The name of the account."""
    value: NotRequired[float]
    r"""The amount or value of the item"""
    items: NotRequired[Any]
    r"""A list of balance sheet accounts"""


class BalanceSheetEquityAccount(BaseModel):
    r"""A balance sheet equity account represents the financial position of a company at a specific point in time."""

    account_id: Optional[str] = None
    r"""The unique identifier for the account."""

    code: Optional[str] = None
    r"""The account code of the account"""

    name: Optional[str] = None
    r"""The name of the account."""

    value: Optional[float] = None
    r"""The amount or value of the item"""

    items: Optional[Any] = None
    r"""A list of balance sheet accounts"""


class BalanceSheetUncategorizedItemsAccountTypedDict(TypedDict):
    r"""A balance sheet uncategorized items account represents the financial position of a company at a specific point in time."""

    account_id: NotRequired[str]
    r"""The unique identifier for the account."""
    code: NotRequired[str]
    r"""The account code of the account"""
    name: NotRequired[str]
    r"""The name of the account."""
    value: NotRequired[float]
    r"""The amount or value of the item"""
    items: NotRequired[Any]
    r"""A list of balance sheet accounts"""


class BalanceSheetUncategorizedItemsAccount(BaseModel):
    r"""A balance sheet uncategorized items account represents the financial position of a company at a specific point in time."""

    account_id: Optional[str] = None
    r"""The unique identifier for the account."""

    code: Optional[str] = None
    r"""The account code of the account"""

    name: Optional[str] = None
    r"""The name of the account."""

    value: Optional[float] = None
    r"""The amount or value of the item"""

    items: Optional[Any] = None
    r"""A list of balance sheet accounts"""


class ReportsTypedDict(TypedDict):
    end_date: str
    r"""The start date of the report"""
    assets: BalanceSheetAssetsAccountTypedDict
    r"""A balance sheet assets account represents the financial position of a company at a specific point in time."""
    liabilities: BalanceSheetLiabilitiesAccountTypedDict
    r"""A balance sheet liabilities account represents the financial position of a company at a specific point in time."""
    equity: BalanceSheetEquityAccountTypedDict
    r"""A balance sheet equity account represents the financial position of a company at a specific point in time."""
    id: NotRequired[str]
    r"""A unique identifier for an object."""
    report_name: NotRequired[str]
    r"""The name of the report"""
    start_date: NotRequired[str]
    r"""The start date of the report"""
    currency: NotRequired[Nullable[Currency]]
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""
    net_assets: NotRequired[float]
    r"""The net assets of the balance sheet"""
    custom_mappings: NotRequired[Nullable[Dict[str, Any]]]
    r"""When custom mappings are configured on the resource, the result is included here."""
    updated_by: NotRequired[Nullable[str]]
    r"""The user who last updated the object."""
    created_by: NotRequired[Nullable[str]]
    r"""The user who created the object."""
    updated_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was last updated."""
    created_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was created."""
    uncategorized_items: NotRequired[BalanceSheetUncategorizedItemsAccountTypedDict]
    r"""A balance sheet uncategorized items account represents the financial position of a company at a specific point in time."""


class Reports(BaseModel):
    end_date: str
    r"""The start date of the report"""

    assets: BalanceSheetAssetsAccount
    r"""A balance sheet assets account represents the financial position of a company at a specific point in time."""

    liabilities: BalanceSheetLiabilitiesAccount
    r"""A balance sheet liabilities account represents the financial position of a company at a specific point in time."""

    equity: BalanceSheetEquityAccount
    r"""A balance sheet equity account represents the financial position of a company at a specific point in time."""

    id: Optional[str] = None
    r"""A unique identifier for an object."""

    report_name: Optional[str] = None
    r"""The name of the report"""

    start_date: Optional[str] = None
    r"""The start date of the report"""

    currency: OptionalNullable[Currency] = UNSET
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""

    net_assets: Optional[float] = None
    r"""The net assets of the balance sheet"""

    custom_mappings: OptionalNullable[Dict[str, Any]] = UNSET
    r"""When custom mappings are configured on the resource, the result is included here."""

    updated_by: OptionalNullable[str] = UNSET
    r"""The user who last updated the object."""

    created_by: OptionalNullable[str] = UNSET
    r"""The user who created the object."""

    updated_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was last updated."""

    created_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was created."""

    uncategorized_items: Optional[BalanceSheetUncategorizedItemsAccount] = None
    r"""A balance sheet uncategorized items account represents the financial position of a company at a specific point in time."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "id",
            "report_name",
            "start_date",
            "currency",
            "net_assets",
            "custom_mappings",
            "updated_by",
            "created_by",
            "updated_at",
            "created_at",
            "uncategorized_items",
        ]
        nullable_fields = [
            "currency",
            "custom_mappings",
            "updated_by",
            "created_by",
            "updated_at",
            "created_at",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class BalanceSheetTypedDict(TypedDict):
    reports: List[ReportsTypedDict]


class BalanceSheet(BaseModel):
    reports: List[Reports]
